{% load betty %}

<section class="video-grid">
  <header>
    <h2>Episodes</h2>
  </header>
  <div class="video-list" id="video-list" data-betty-url="{{ BETTY_ONION_STUDIOS_IMAGE_URL }}" data-count="{{ count }}" data-source="{{ VIDEOHUB_BASE_URL }}/{{ source }}"></div>
</section>